/*
 * Copyright (C) 2011 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by Alex Launi <alex.launi@canonical.com>
 *
 */

namespace Unity.MusicLens {
 	
 	// FIXME there is so much duplicated code in the different search functions.
 	// I am positive this can be simplified. I EXPECT BETTER.
	public class BansheeScopeProxy : GLib.Object
	{
		public Unity.Scope Scope { get; private set; }

		private bool is_dirty;
		private LensSearch? previous_search;
		private BansheeCollection collection;

		construct {
			Scope = new Unity.Scope ("/com/canonical/unity/scope/banshee");
			Scope.search_in_global = true;
			Scope.activate_uri.connect (activate);

			try {
				collection = new BansheeCollection ();
			} catch (DatabaseError error) {
				printerr ("FUCK NO DB HA\n");
				return;
			}

			is_dirty = true;
			previous_search = null;

			/* Listen for filter changes */
			Scope.filters_changed.connect (
				() => {          
					if (Scope.active_search != null)
					{
						is_dirty = true;
						Scope.notify_property ("active-search");
					}
				}
			);

			/* Listen for changes to the lens entry search */
			Scope.notify["active-search"].connect (
				(obj, pspec) => {
					var search = Scope.active_search;

					if (!(is_dirty || search_has_really_changed (previous_search, search)))
						return;

					is_dirty = false;

					if (search_is_invalid (search))
					{
						update_without_search_async.begin(search);
					}
					else
					{
						update_search_async.begin (search);
					}
					previous_search = search;
				}
			);

			/* Listen for changes to the global search */
			Scope.notify["active-global-search"].connect (
				(obj, pspec) => {

					var search = Scope.active_global_search;

					if (search_is_invalid (search))
						return;

					if (!search_has_really_changed (previous_search, search))
						return;

					update_global_search_async.begin (search);
					previous_search = search;
				}
			);
		}

		/**
		 * Tells banshee to play the selected uri(s)
		 */
		public Unity.ActivationResponse activate (string uri)
    {
      debug (@"Activating: $uri");
      try {
        GLib.Process.spawn_async (null, 
	        											  {"banshee", "--play", uri}, 
	        											  null, 
	        											  GLib.SpawnFlags.SEARCH_PATH,
        													null, 
        													null);
        return new Unity.ActivationResponse (Unity.HandledType.HIDE_DASH);
      } catch (GLib.SpawnError error) {
        warning ("Failed to launch URI %s", uri);
        return new Unity.ActivationResponse (Unity.HandledType.NOT_HANDLED);
      }
    }

    // FIXME needs you know .. WRITTEN.
    /* based on the filters(?) get the default results to show when the music lens is
    	just opened without filters, like when someone hits Super+M
		*/
    private async void update_without_search_async (LensSearch search)
    {
      var results_model = Scope.results_model;

      /* Prevent concurrent searches and concurrent updates of our models,
       * by preventing any notify signals from propagating to us.
       * Important: Remeber to thaw the notifys again! */
      Scope.freeze_notify ();

      // FIXME - IE IMPLEMENT ME
      // so here we just get NUM_RESULTS_DEFAULT tracks from the banshee CoreTracks db
      /*
     	a_bunch_of_tracks = banshee-db.get_some_tracks ();
      results_model.clear ();
      add_the_results_to_our_model(a_bunch_of_tracks, results_model)
      */
      results_model.clear ();
      collection.search (search, results_model, 100);
      
      /* Allow new searches once we enter an idle again.
       * We don't do it directly from here as that could mean we start
       * changing the model even before we had flushed out current changes
       */
      Idle.add (() => {
        Scope.thaw_notify ();
        return false;
      });
      
      search.finished ();
    }

    /**
     * results for a global search from just hitting Super. here we want to return
     * a smaller number of results with 0 filters.
     */
    private async void update_global_search_async (LensSearch search)
    {
      var results_model = Scope.global_results_model;

      if (search_is_invalid (search))
        {
          results_model.clear ();
          return;
        }
      
      /* Prevent concurrent searches and concurrent updates of our models,
       * by preventing any notify signals from propagating to us.
       * Important: Remeber to thaw the notifys again! */
      Scope.freeze_notify ();
      
      
    	/* get a bunch of tracks, whatever just get some we'll figure out how
    	to pick later */
    	// some_tracks = get_some_tracks_ha ();
      results_model.clear ();
      collection.search (search, results_model, 20);
      // add_the_tracks_to_the_results_model (some_tracks, results_model);
      
      /* Allow new searches once we enter an idle again.
       * We don't do it directly from here as that could mean we start
       * changing the model even before we had flushed out current changes
       */
      Idle.add (() => {
        Scope.thaw_notify ();
        return false;
      });
      
      search.finished ();
    }

    private async void update_search_async (LensSearch search)
    {
    	// just pretend like there's no search
      if (search_is_invalid (search))
        {
          update_without_search_async.begin (search);
          return;
        }

      var results_model = Scope.results_model;

      /* Prevent concurrent searches and concurrent updates of our models,
       * by preventing any notify signals from propagating to us.
       * Important: Remeber to thaw the notifys again! */
      Scope.freeze_notify ();

      // umm do the exact same thing as always it seems.. but with a different
      // number of results. what the fuck who wrote all of this crap! I am going
      // to make mine bee-you-ti-full and fasssssssssssssst
      results_model.clear ();
      collection.search (search, results_model, 50);
      
      /* Allow new searches once we enter an idle again.
       * We don't do it directly from here as that could mean we start
       * changing the model even before we had flushed out current changes
       */
      Idle.add (() => {
        Scope.thaw_notify ();
        return false;
      });
      
      search.finished ();
    }

    private bool search_is_invalid (LensSearch? search)
    {
      /* This boolean expression is unfolded as we seem to get
       * some null dereference if we join them in a big || expression */
      if (search == null)
        return true;
      else if (search.search_string == null)
        return true;
      
      return search.search_string.strip() == "";
    }

  	private bool search_has_really_changed (LensSearch? old_search,
	                                          LensSearch? new_search)
	  {
	    if (old_search == null && new_search == null)
	      return false;
	    
	    string s1, s2;
	   
	    if (old_search == null)
	      {
	        s1 = new_search.search_string;
	        if (s1 == null || s1.strip() == "")
	          return false;
	        else
	          return true;
	      }
	    else if (new_search == null)
	      {
	        s2 = old_search.search_string;
	        if (s2 == null || s2.strip() == "")
	          return false;
	        else
	          return true;
	      }
	    else
	      {
	        s1 = new_search.search_string;
	        s2 = old_search.search_string;
	        if (s1 == null)
	          {
	            if (s2 == null || s2.strip() == "")
	              return false;
	            else
	              return true;
	          }
	        else if (s2 == null)
	          {
	            if (s1 == null || s1.strip() == "")
	              return false;
	            else
	              return true;
	          }
	        else
	          return s1.strip () != s2.strip ();
	      }
	  }
	}
}