/*
 * Copyright (C) 2010 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by Mikkel Kamstrup Erlandsen <mikkel.kamstrup@canonical.com>
 *
 */
using Dee;
using Config;
using Gee;

namespace Unity.MusicLens {
  
  const string ICON_PATH = Config.DATADIR + "/icons/unity-icon-theme/lenses/svg/";
  
  public class Daemon : GLib.Object
  {
    private Unity.Lens lens;
    private BansheeScopeProxy banshee;
     
    construct
    {
      banshee = new BansheeScopeProxy ();

      lens = new Unity.Lens("/com/canonical/unity/lens/music", "music");
      lens.search_in_global = true;
      lens.search_hint = _("Search Music Collection");
      lens.visible = true;
      
      populate_categories ();
      populate_filters();

      lens.add_local_scope (banshee.Scope);

      try {
        lens.export ();
      } catch (GLib.IOError e) {
        stdout.printf ("error %s\n", e.message);
      }
    }

    private void populate_filters ()
    {
      Unity.Filter[] filters = {};

      /* Decade filter */
      {
        var filter = new MultiRangeFilter ("decade", _("Decade"));

        filter.add_option ("old", _("Old"));
        filter.add_option ("60s", _("60s"));
        filter.add_option ("70s", _("70s"));
        filter.add_option ("90s", _("80s"));
        filter.add_option ("00s", _("00s"));
        filter.add_option ("10s", _("10s"));

        filters += filter;
      }

      /* Genre filter */
      {
        var filter = new CheckOptionFilter ("genre", _("Genre"));

        filter.add_option ("blues", _("Blues"));
        filter.add_option ("classic", _("Classic"));
        filter.add_option ("country", _("Country"));
        filter.add_option ("disco", _("Disco"));
        filter.add_option ("funk", _("funk"));
        filter.add_option ("rock", _("rock"));
        filter.add_option ("metal", _("Metal"));
        filter.add_option ("hip-hop", _("Hip-hop"));
        filter.add_option ("house", _("House"));
        filter.add_option ("new-wave", _("New-wave"));
        filter.add_option ("r-and-b", _("R&B"));
        filter.add_option ("punk", _("Punk"));
        filter.add_option ("jazz", _("Jazz"));
        filter.add_option ("pop", _("Pop"));
        filter.add_option ("reggae", _("Reggae"));
        filter.add_option ("soul", _("Soul"));
        filter.add_option ("techno", _("Techno"));
        filter.add_option ("other", _("Other"));

        filters += filter;
      }

      lens.filters = filters;
    }

    // FIXME icons
    private void populate_categories ()
    {
      Unity.Category[] categories = {};

      var cat = new Unity.Category (_("Songs"), ICON_PATH + "category-recent.svg");
      categories += cat;

      cat =  new Unity.Category (_("Albums"), ICON_PATH + "category-downloads.svg");
      categories += cat;

      cat = new Unity.Category (_("Available for Purchase"), ICON_PATH + "category-favoritefolders.svg");
      categories += cat;

      lens.categories = categories;
    }
  }
} /* namespace */

